#! /usr/bin/Rscript

library(plyr)
library(reshape)
library(ggplot2)

# Data set sorted by experimental condition (stock, libxul, static)
# and then in increasing order of hard drive performance.  Discard the
# first data point in each set, which is nonrepresentative because it
# does some setup work that is cached for later runs, as well as
# priming operating-system and CPU caches.

timings.raw <- data.frame(dynamic.4200=scan('data-9.10-4200rpm-USB/stock.csv', skip=1),
                          dynamic.5400=scan('data-9.10-5400rpm/stock.csv', skip=1),
                          dynamic.7200=scan('data-9.10-7200rpm/stock.csv', skip=1),
                          dynamic.ssd=scan('data-9.10-ssd/stock.csv', skip=1),
                          libxul.4200=scan('data-9.10-4200rpm-USB/libxul.csv', skip=1),
                          libxul.5400=scan('data-9.10-5400rpm/libxul.csv', skip=1),
                          libxul.7200=scan('data-9.10-7200rpm/libxul.csv', skip=1),
                          libxul.ssd=scan('data-9.10-ssd/libxul.csv', skip=1),
                          static.4200=scan('data-9.10-4200rpm-USB/static.csv', skip=1),
                          static.5400=scan('data-9.10-5400rpm/static.csv', skip=1),
                          static.7200=scan('data-9.10-7200rpm/static.csv', skip=1),
                          static.ssd=scan('data-9.10-ssd/static.csv', skip=1)
                          )

timings.flat <- melt(timings.raw, id.vars=c(), variable_name='condition')

# compute mean and standard deviation in each condition
timings.norm <- ddply(timings.flat, .(condition), transform,
                      value = (value - mean(value)) / sd(value))

# overlay frame with the normal distribution
overlay.norm <- data.frame(x=seq(-3.5,3.5,0.1), y=dnorm(seq(-3.5,3.5,0.1)))

# diagnostic plot of the pdf of each condition
p <- ggplot(timings.norm, aes(x=value)) + facet_wrap(~ condition, ncol=4) +
  geom_density(fill='gray', colour=NA) + geom_rug() +
  layer(data=overlay.norm, mapping=aes(x=x, y=y), geom='line',
        geom_params=list(colour='red', size=0.5)) +
  scale_x_continuous(name='Startup time, normalized per condition',
                     limits=c(-9, 9), breaks=seq(-9, 9, 3)) +
  ylab('Observation density')

pdf(file='diagnosis-u.pdf', width=9, height=6.5)
print(p)
dev.off()
system('qpdf diagnosis-u.pdf diagnosis.pdf')
